#
# Copyright (c) 2017, Stephanie Wehner and Axel Dahlberg
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#    This product includes software developed by Stephanie Wehner, QuTech.
# 4. Neither the name of the QuTech organization nor the
#    names of its contributors may be used to endorse or promote products
#    derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER ''AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import os
my_path = os.path.abspath(os.path.dirname(__file__))

os.environ["NETSIM"] = os.path.join(my_path, "SimulaQron")
os.environ["PYTHONPATH"] = my_path + ':$PYTHONPATH'



from SimulaQron.general.hostConfig import *
from SimulaQron.cqc.backend.cqcHeader import *
from SimulaQron.cqc.pythonLib.cqc import *
from SimulaQron.toolbox.measurements import parity_meas

import random


import sys


#####################################################################################################
#
# main
#
def main():

	# Initialize the connection
    Alice = CQCConnection("Alice")

	# Create EPR pairs
    q1 = Alice.createEPR("Bob")
    q2 = Alice.createEPR("Bob")

	# Make sure we order the qubits consistently with Bob
	# Get entanglement IDs
    q1_ID = q1.get_entInfo().id_AB
    q2_ID = q2.get_entInfo().id_AB

    if q1_ID < q2_ID:
        qa = q1
        qc = q2
    else:
        qa = q2
        qc = q1

	# Get row
    row = int(sys.argv[1])

    file_exists = os.path.isfile('./files/Strategy.txt')
    if file_exists:
        fh = open('./files/Strategy.txt', "r")
        data = fh.read().splitlines()
        p00 = data[0]; f00 = data[1]; s00 = data[2]
        p01 = data[3]; f01 = data[4]; s01 = data[5]
        p02 = data[6]; f02 = data[7]; s02 = data[8]

        p10 = data[9]; f10 = data[10]; s10 = data[11]
        p11 = data[12]; f11 = data[13]; s11 = data[14]
        p12 = data[15]; f12 = data[16]; s12 = data[17]

        p20 = data[18]; f20 = data[19]; s20 = data[20]
        p21 = data[21]; f21 = data[22]; s21 = data[23]
        p22 = data[24]; f22 = data[25]; s22 = data[26]

        g00 = f00 + s00; g01 = f01 + s01; g02 = f02 + s02
        g10 = f10 + s10; g11 = f11 + s11; g12 = f12 + s12
        g20 = f20 + s20; g21 = f21 + s21; g22 = f22 + s22 
    
    # if strategy is not created, use winning strategy
    else:
        p00 = '+'; p01 = '+'; p02 = '+'
        p10 = '-'; p11 = '+'; p12 = '-'
        p20 = '+'; p21 = '+'; p22 = '+'

        g00 = "XI"; g01 = "XX"; g02 = "IX"
        g10 = "XZ"; g11 = "YY"; g12 = "ZX"
        g20 = "IZ"; g21 = "ZZ"; g22 = "ZI"
    
    
	# Perform the three measurements
    if row == 1:
        if p00 == '+':
            m0 = parity_meas([qa, qc], g00, Alice)
        else:
            m0 = parity_meas([qa, qc], g00, Alice, negative=True)

        if p01 == '+':
            m1 = parity_meas([qa, qc], g01, Alice)
        else:
            m1 = parity_meas([qa, qc], g01, Alice, negative=True)

        if p02 == '+':
            m2 = parity_meas([qa, qc], g02, Alice)
        else:
            m2 = parity_meas([qa, qc], g02, Alice, negative=True)

    elif row == 2:
        if p10 == '+':
            m0 = parity_meas([qa, qc], g10, Alice)
        else:
            m0 = parity_meas([qa, qc], g10, Alice, negative=True)

        if p11 == '+':
            m1 = parity_meas([qa, qc], g11, Alice)
        else:
            m1 = parity_meas([qa, qc], g11, Alice, negative=True)

        if p12 == '+':
            m2 = parity_meas([qa, qc], g12, Alice)
        else:
            m2 = parity_meas([qa, qc], g12, Alice, negative=True)

    elif row == 3:
        if p20 == '+':
            m0 = parity_meas([qa, qc], g20, Alice)
        else:
            m0 = parity_meas([qa, qc], g20, Alice, negative=True)

        if p21 == '+':
            m1 = parity_meas([qa, qc], g21, Alice)
        else:
            m1 = parity_meas([qa, qc], g21, Alice, negative=True)

        if p22 == '+':
            m2 = parity_meas([qa, qc], g22, Alice)
        else:
            m2 = parity_meas([qa, qc], g22, Alice, negative=True)

    else:
        m0 = 0
        m1 = 0
        m2 = 0

	
    if not os.path.exists('files'):
        os.makedirs('files')
    f = open("./files/AliceSolution.txt", "w")
    f.write("{}\n".format(row))
    f.write("{}\n".format(m0))
    f.write("{}\n".format(m1))
    f.write("{}\n".format(m2))

	# Clear qubits
    qa.measure()
    qc.measure()

	# Stop the connections
    Alice.close()
	
    return


##################################################################################################
main()

