var {PythonShell} = require('python-shell')
const path = require('path')
const BrowserWindow = require('electron').remote.BrowserWindow;
const url = require('url');

const swal = require('sweetalert2')

var AliceRow = 0;
var BobCol = 0;

var AliceMessage = '';
var BobMessage = '';

// When Assign button is clicked
const RowColumnAssign = document.getElementById('RowColumnAssign');
if(RowColumnAssign) {
    RowColumnAssign.addEventListener('click', function (event) {

        var fs = require('fs');
        // If Strategy has not been set by Alice and Bob
        if (!fs.existsSync('./files/Strategy.txt')) {
            swal({
                title: "Alice and Bob have not yet decided a strategy. Go back to the main window to set strategy.",
                type: 'error',
                confirmButtonText: 'Ok'
            });
            return;
        }

        var t = document.getElementById("AliceRow").value;
        if(t == '1' || t == '2' || t == '3') {
            AliceRow = parseInt(t);
        } else {
            swal({
                title: "Enter row number for Alice between 1 and 3",
                type: 'error',
                confirmButtonText: 'Ok'
            });
            return;
        }
    
        t = document.getElementById("BobColumn").value;
        if(t == '1' || t == '2' || t == '3') {
            BobCol = parseInt(t);
        } else {
            swal({
                title: "Enter column number for Bob between 1 and 3",
                type: 'error',
                confirmButtonText: 'Ok'
            });
            return;
        }
    
        console.log('Alice row = ' + AliceRow);
        console.log('Bob col = ' + BobCol);
    
        var aliceOptions = {
        scriptPath : path.join(__dirname, '../../engine/'),
        args : [AliceRow]
        }
    
        var alice = new PythonShell('alice.py', aliceOptions);
    
        var bobOptions = {
            scriptPath : path.join(__dirname, '../../engine/'),
            args : [BobCol]
        }
        
        var bob = new PythonShell("bob.py", bobOptions);
    
        alice.on('message', function (message) {
            console.log(message);
        });      

        bob.on('message', function(message) {
            console.log(message);
        });

        const CheckAnswer = document.getElementById('CheckAnswer');
        if(CheckAnswer) {
            CheckAnswer.style.display = "block";
        }

        // Hide all other elements
        document.getElementById('ResultHead').style.display = "none";
        document.getElementById('AliceAnswerHead').style.display = "none";
        document.getElementById('AliceTable').style.display = "none";
        document.getElementById('BobAnswerHead').style.display = "none";
        document.getElementById('BobTable').style.display = "none";
        RowColumnAssign.style.display = "none";

        // Remove previous game's Alice's and Bob's attempted answer files
        var fs = require('fs-extra');
        fs.removeSync('./files/AliceAttempt.txt'); 
        fs.removeSync('./files/BobAttempt.txt'); 

        swal({
            title: "Judge, you have assigned Row " + AliceRow + " to Alice and Column " + BobCol + " to Bob.",
            type: 'info',
            confirmButtonText: 'Cool'
        });

    });
}

// To check answers of Alice and Bob
const CheckAnswer = document.getElementById('CheckAnswer');
if(CheckAnswer) {
    CheckAnswer.addEventListener('click', function(event) {

        var fs = require('fs');
        // If Alice has not yet submitted answer
        if (!fs.existsSync('./files/AliceAttempt.txt')) {
            swal({
                title: "Alice has not yet submitted the answer.",
                type: 'error',
                confirmButtonText: 'Ok'
            });
            return;
        }

        // Read Alice's answer
        var alice_m = fs.readFileSync('./files/AliceAttempt.txt','utf8').split('\n');

        // If Bob has not yet submitted answer
        if (!fs.existsSync('./files/BobAttempt.txt')) {
            swal({
                title: "Bob has not yet submitted the answer.",
                type: 'error',
                confirmButtonText: 'Ok'
            });
            return;
        }

        // Read Bob's answer
        var bob_m = fs.readFileSync('./files/BobAttempt.txt','utf8').split('\n');
        
        console.log (alice_m);
        console.log (bob_m);
        if(alice_m[BobCol-1] == bob_m[AliceRow-1]) {
            swal({
                title: "Alice and Bob win the Game",
                type: 'success',
                confirmButtonText: 'Cool'
            });
        } else {
            swal({
                title: "Alice and Bob lose the Game",
                type: 'error',
                confirmButtonText: 'Ok'
            });
        }
        if(alice_m[BobCol-1] == bob_m[AliceRow-1]) {
            document.getElementById('ResultHead').innerHTML = "Alice and Bob win the game!"
        } else {
            document.getElementById('ResultHead').innerHTML = "Alice and Bob lose the game!"
        }
        UpdateAliceTable(alice_m);
        UpdateBobTable(bob_m);

        document.getElementById('ResultHead').style.display = "block";
        document.getElementById('AliceAnswerHead').style.display = "block";
        document.getElementById('AliceTable').style.display = "block";
        document.getElementById('BobAnswerHead').style.display = "block";
        document.getElementById('BobTable').style.display = "block";
        document.getElementById('RowColumnAssign').style.display = "block";

        // Remove Alice's and Bob's measurement solution after checking their answers
        var fs = require('fs-extra');
        fs.removeSync('./files/AliceSolution.txt'); 
        fs.removeSync('./files/BobSolution.txt'); 

    });   
}

// Alice's table with answers
function UpdateAliceTable(alice_m) {
    var matrix = [];
    for(var i=0; i<9; i++) {
        matrix[i] = [];
        for(var j=0; j<9; j++) {
            matrix[i][j] = '-';
        }
    }

    for(var i = 0; i < 3; i++) {
        matrix[AliceRow-1][i] = alice_m[i];
    }

    document.getElementById('Am00').innerHTML = matrix[0][0];
    document.getElementById('Am01').innerHTML = matrix[0][1];
    document.getElementById('Am02').innerHTML = matrix[0][2];
    document.getElementById('Am10').innerHTML = matrix[1][0];
    document.getElementById('Am11').innerHTML = matrix[1][1];
    document.getElementById('Am12').innerHTML = matrix[1][2];
    document.getElementById('Am20').innerHTML = matrix[2][0];
    document.getElementById('Am21').innerHTML = matrix[2][1];
    document.getElementById('Am22').innerHTML = matrix[2][2];

}

// Bob's table with answers
function UpdateBobTable(bob_m) {
    var matrix = [];
    for(var i=0; i<9; i++) {
        matrix[i] = [];
        for(var j=0; j<9; j++) {
            matrix[i][j] = '-';
        }
    }

    for(var i = 0; i < 3; i++) {
        matrix[i][BobCol-1] = bob_m[i];
    }

    document.getElementById('Bm00').innerHTML = matrix[0][0];
    document.getElementById('Bm01').innerHTML = matrix[0][1];
    document.getElementById('Bm02').innerHTML = matrix[0][2];
    document.getElementById('Bm10').innerHTML = matrix[1][0];
    document.getElementById('Bm11').innerHTML = matrix[1][1];
    document.getElementById('Bm12').innerHTML = matrix[1][2];
    document.getElementById('Bm20').innerHTML = matrix[2][0];
    document.getElementById('Bm21').innerHTML = matrix[2][1];
    document.getElementById('Bm22').innerHTML = matrix[2][2];
}


